#!/usr/bin/perl

# GPRename 2.0
# Home Page : http://gprename.sourceforge.net/en
# SVN       : http://gprename.svn.sourceforge.net/viewvc/gprename
# Copyright (C) 2001-2006
# See the README file for the Credits and the License
# GTK2-Perl documentations, helps, tutorials :
# http://gtk2-perl.sourceforge.net/doc


######################################################################
# Initialisation                                                     #
######################################################################

use Gtk2 -init;
use Gtk2::SimpleList;         # for the tree and the file/dir list
use File::Spec::Functions;    # for the tree
use Cwd;                      # to get the current directory
use Encode qw(decode encode); # to handle accents
use Fcntl ':flock';           # for the flock function in &read_settings
use strict;                   # require variables to be declared before using them and distrust barewords
use warnings;                 # print warning messages on the command line
use Glib qw/TRUE FALSE/;      # to use the bareword TRUE and FALSE

# Change the font globally
Gtk2::Rc->parse_string(<<__);
style "my_text" {
   font_name ="courier new"
}
widget "*" style "my_text"
__


# Declaration of global variables
my $recursive=0;              # option : show files/folders recursively (true=1, false=0)
my @selected_items;           # array of selected rows of files/folders
my $install_dir = $ARGV[0];   # come without a trailing slash
our %langs;
my %settings;
my $setting_file = "$ENV{'HOME'}/.gprename";
my $log_file = "$ENV{'HOME'}/.gprename_log";
my @undo_oldname = ();
my @undo_newname = ();
my $max_length;
my $max_length_undo;
my $security=1;
my $exist=0;
my @selection;

our $str_button_apply;
our $str_button_preview;
our $str_button_undo;
our $str_case_all_lo;
our $str_case_all_up;
our $str_case_first;
our $str_exists1;
our $str_exists2;
our $str_gprename_version = 'GPRename 2.0';
our $str_insertdelete_at;
our $str_insertdelete_between;
our $str_insertdelete_delete;
our $str_insertdelete_insert;
our $str_list_filename;
our $str_list_newname;
our $str_log_size;
our $str_menu_about;
our $str_menu_directory;
our $str_menu_file;
our $str_menu_help;
our $str_menu_lang;
our $str_menu_options;
our $str_menu_quit;
our $str_menu_tips;
our $str_menu_log;
our $str_menu_log_clear;
our $str_notebook_casechange;
our $str_notebook_insertdelete;
our $str_notebook_numerical;
our $str_notebook_replaceremove;
our $str_numeric_increment;
our $str_numeric_keep_filenames1;
our $str_numeric_keep_filenames2;
our $str_numeric_keep_filenames3;
our $str_numeric_prefix;
our $str_numeric_start;
our $str_numeric_suffix;
our $str_options_auto_preview;
our $str_options_fullscreen;
our $str_options_recursively;
our $str_options_security;
our $str_options_trim_spaces;
our $str_options_zero_autofill;
our $str_replace_case;
our $str_replace_regex;
our $str_replace;
our $str_replace_with;
our $str_tips;

# Load settings file : "languages/langs.pl" and "~/.gprename" and "languages/language_choosen"
require "$install_dir/languages/langs.pl";
$settings{'lang'} = 'English';
$settings{'trim_spaces'}   = TRUE;
$settings{'auto_preview'}  = 0;
$settings{'zero_autofill'} = TRUE;
$settings{'fullscreen'}    = 0;
$settings{'width'}         = 640;
$settings{'height'}        = 480;
$settings{'security'}      = 0;
&read_settings;
do "$install_dir/languages/" . $langs{$settings{'lang'}};


######################################################################
# Main window                                                        #
######################################################################

# Create a window
my $window = new Gtk2::Window('toplevel');
$window->set_position('none');
$window->set_title( $str_gprename_version );
$window->resize ($settings{'width'}, $settings{'height'});
$window->signal_connect( "delete_event", \&quit );
$window->show();

# Set window icon
my $style;
my $pixmap;
my $mask;
my $icon = "$install_dir/pixmaps/gprename_icon64.xpm";
$style = $window->get_style()->bg('normal');
( $pixmap, $mask ) = Gtk2::Gdk::Pixmap->create_from_xpm( $window->window, $style, $icon );
$window->window->set_icon( $window->window, $pixmap, $mask );

# Create a vertical main pane
my $main_paned = new Gtk2::VPaned();
$window->add($main_paned);

# Create a vbox for menubar and pane
my $top_vbox = new Gtk2::VBox( FALSE, 0 );
$main_paned->pack1( $top_vbox, TRUE, TRUE );


######################################################################
# Menubar                                                            #
######################################################################

# Create the menubar
my $menubar = new Gtk2::MenuBar();
$top_vbox->pack_start( $menubar, FALSE, FALSE, 0 );

# File
my $file_mm = new Gtk2::MenuItem( $str_menu_file );
$menubar->append( $file_mm );

my $file_menu = new Gtk2::Menu();
$file_mm->set_submenu( $file_menu );

# File - Quit
my $quit_mi = new Gtk2::MenuItem( $str_menu_quit );
$file_menu->append($quit_mi);
$quit_mi->signal_connect( 'activate', \&quit );

# Options
my $options_mi = new Gtk2::MenuItem( $str_menu_options );
$menubar->append( $options_mi );
my $options_menu = new Gtk2::Menu();
$options_mi->set_submenu( $options_menu );

# Options - Recursive renaming
#my $recursive_rmi = new Gtk2::CheckMenuItem( $str_options_recursively );
#$options_menu->append( $recursive_rmi );
#$recursive_rmi->set_active( FALSE );
#$recursive_rmi->signal_connect( 'activate', \&recursive );

# Options - Automatically trim spaces at the beginning
my $trim_spaces_rmi = new Gtk2::CheckMenuItem( $str_options_trim_spaces );
$options_menu->append( $trim_spaces_rmi );
if   ( $settings{'trim_spaces'} ) { $trim_spaces_rmi->set_active( TRUE  ); }
else                                       { $trim_spaces_rmi->set_active( FALSE ); }
$trim_spaces_rmi->signal_connect( 'activate', \&options_trim_spaces );

# Options - Zero auto-fill
my $zero_autofill_rmi = new Gtk2::CheckMenuItem( $str_options_zero_autofill );
$options_menu->append( $zero_autofill_rmi );
if   ( $settings{'zero_autofill'} ) { $zero_autofill_rmi->set_active( TRUE  ); }
else                                         { $zero_autofill_rmi->set_active( FALSE ); }
$zero_autofill_rmi->signal_connect( 'activate', \&options_zero_autofill );

# Options - Fullscreen
my $fullscreen_rmi = new Gtk2::CheckMenuItem( $str_options_fullscreen );
$options_menu->append( $fullscreen_rmi );
if   ( $settings{'fullscreen'} ) {  $fullscreen_rmi->set_active( TRUE  ); $window->fullscreen; }
else                                      { $fullscreen_rmi->set_active( FALSE ); $window->unfullscreen; }
$fullscreen_rmi->signal_connect( 'activate', \&options_fullscreen );

# Options - Automatic preview
my $auto_preview_rmi = new Gtk2::CheckMenuItem( $str_options_auto_preview );
$options_menu->append( $auto_preview_rmi );
if   ( $settings{'auto_preview'} ) { $auto_preview_rmi->set_active( TRUE  ); }
else                                        { $auto_preview_rmi->set_active( FALSE ); }
$auto_preview_rmi->signal_connect( 'activate', \&options_auto_preview );

# Options - View Log
my $view_log_mi = new Gtk2::MenuItem( $str_menu_log );
$options_menu->append( $view_log_mi );
$view_log_mi->signal_connect( 'activate', \&options_view_log );

# Options - Disable security check
my $security_rmi = new Gtk2::CheckMenuItem( $str_options_security );
$options_menu->append( $security_rmi );
$security_rmi->set_active( FALSE );
$security_rmi->signal_connect( 'activate', \&options_security );

# Language
my $lang_menu = new Gtk2::Menu();
my $lang_mm = new Gtk2::MenuItem( $str_menu_lang );
$menubar->append( $lang_mm );
$lang_mm->set_submenu( $lang_menu );

my $lang_rmi;
my $lang_rmi_start;
my $lang;
foreach $lang ( sort keys %langs ) {
   $lang_rmi = Gtk2::RadioMenuItem->new_with_label ( $lang_rmi, $lang );
   $lang_menu->append($lang_rmi);
   if ( $lang eq $settings{'lang'} ) { $lang_rmi_start = $lang_rmi; }
   $lang_rmi->signal_connect( 'activate', \&language_selected );
}

# Help
my $help_mm = new Gtk2::MenuItem( $str_menu_help );
$menubar->append($help_mm);
my $help_menu = new Gtk2::Menu();
$help_mm->set_submenu($help_menu);

# Help - Tips
my $tips_mi = new Gtk2::MenuItem( $str_menu_tips );
$help_menu->append($tips_mi);
$tips_mi->signal_connect( 'activate', \&help_tips );

# Help - About
my $about_mi = new Gtk2::MenuItem( $str_menu_about );
$help_menu->append($about_mi);
$about_mi->signal_connect( 'activate', \&help_about );


######################################################################
# Tree and file/folder list                                          #
######################################################################

# First we create the Notebook of files/dirs
my $file_dir_notebook = new Gtk2::Notebook();
$file_dir_notebook->set_current_page (0);

my $filelist_sw = new Gtk2::ScrolledWindow( undef, undef );
$filelist_sw->set_policy( 'automatic', 'automatic' );
my $filelist = Gtk2::SimpleList->new( $str_list_filename => 'text', $str_list_newname => 'text' ); 
$filelist->get_selection->set_mode ('multiple');
map { $_->set_resizable(TRUE) } $filelist->get_columns;
$filelist->get_selection->signal_connect( 'changed', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
$filelist_sw->add( $filelist );
$str_menu_file =~ s/_//;
$file_dir_notebook->append_page( $filelist_sw, $str_menu_file );
$filelist->set_rules_hint (TRUE); # property to draw rows in alternate colors

my $dirlist_sw = new Gtk2::ScrolledWindow( undef, undef );
$dirlist_sw->set_policy( 'automatic', 'automatic' );
my $dirlist = Gtk2::SimpleList->new( $str_list_filename => 'text', $str_list_newname => 'text' ); 
$dirlist->get_selection->set_mode ('multiple');
map { $_->set_resizable(TRUE) } $dirlist->get_columns;
$dirlist->get_selection->signal_connect( 'changed', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
$dirlist_sw->add( $dirlist );
$file_dir_notebook->append_page( $dirlist_sw, $str_menu_directory );
$dirlist->set_rules_hint (TRUE); # property to draw rows in alternate colors

$file_dir_notebook->signal_connect_after( 'switch-page', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );

# Secondly, we create the Tree
# These lines and the functions that comes with it came from
# http://mail.gnome.org/archives/gtk-perl-list/2006-October/msg00003.html
my $dirmodel = Gtk2::TreeStore->new( qw( Glib::String ) );
my $child = $dirmodel->append( undef );
$dirmodel->set( $child, 0 => '/' );
my $dirtree = Gtk2::TreeView->new( $dirmodel );
$dirtree->append_column( Gtk2::TreeViewColumn->new_with_attributes( '', Gtk2::CellRendererText->new(), text => 0 ) );
$dirtree->set_headers_visible( FALSE );
$dirtree->signal_connect( 'cursor_changed', \&list_files_dirs );
$dirtree->signal_connect( 'row_expanded', \&list_dir_in_tree );
my $sw = Gtk2::ScrolledWindow->new;
$sw->set_policy( qw( automatic automatic ) );
$sw->add_with_viewport( $dirtree );
populate_node( $dirmodel, $dirmodel->get_iter_first );

# And third we create an HPaned and put the Tree and Notebook in it
my $hpaned = Gtk2::HPaned->new; 
$top_vbox->pack_end( $hpaned, TRUE, TRUE, 2 );
$hpaned->pack1( $sw, TRUE, TRUE );
$hpaned->pack2( $file_dir_notebook, TRUE, TRUE );


######################################################################
# Create manipulation items                                          #
######################################################################

my $table = new Gtk2::Table( 1, 5, TRUE );
$main_paned->pack2( $table, FALSE, TRUE );

my $notebook = new Gtk2::Notebook();
$notebook->set_current_page (0);
$table->attach_defaults( $notebook, 0, 4, 0, 1 );
$notebook->signal_connect_after( 'switch-page', sub { if ( $settings{'auto_preview'} ) { &preview; } } );


######################################################################
# Case change                                                        #
######################################################################

my $case_vbox = new Gtk2::VBox(  FALSE, 4 );
my $case_hbox1 = new Gtk2::HBox( FALSE, 4 );
my $case_hbox2 = new Gtk2::HBox( FALSE, 4 );
my $case_hbox3 = new Gtk2::HBox( FALSE, 4 );
my $case_label = new Gtk2::Label( $str_notebook_casechange );
$notebook->append_page( $case_vbox, $case_label );

my $case_all_up_radio = new Gtk2::RadioButton( undef, $str_case_all_up );
my $case_all_lo_radio = new Gtk2::RadioButton( $case_all_up_radio, $str_case_all_lo );
my $case_first_radio = new Gtk2::RadioButton( $case_all_up_radio, $str_case_first );
$case_all_up_radio->signal_connect( 'clicked', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
$case_all_lo_radio->signal_connect( 'clicked', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
$case_first_radio->signal_connect( 'clicked', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $case_first_after_entry = Gtk2::Entry->new_with_max_length(254);
$case_first_after_entry->set_text(' _-\([');

$case_vbox->pack_start( $case_hbox1,              FALSE, FALSE, 4 );
$case_vbox->pack_start( $case_hbox2,              FALSE, FALSE, 4 );
$case_vbox->pack_start( $case_hbox3,              FALSE, FALSE, 4 );
$case_hbox1->pack_start( $case_all_up_radio,      FALSE, FALSE, 4 );
$case_hbox2->pack_start( $case_all_lo_radio,      FALSE, FALSE, 4 );
$case_hbox3->pack_start( $case_first_radio,       FALSE, FALSE, 4 );
$case_hbox3->pack_start( $case_first_after_entry, FALSE, FALSE, 4 );


######################################################################
# Insert / Delete                                                    #
######################################################################

my $insdel_vbox  = new Gtk2::VBox( FALSE, 4 );
my $insdel_hbox1 = new Gtk2::HBox( FALSE, 4 );
my $insdel_hbox2 = new Gtk2::HBox( FALSE, 4 );
my $insdel_label = new Gtk2::Label( $str_notebook_insertdelete);
$notebook->append_page( $insdel_vbox, $insdel_label );

# in hbox1
my $insert_radio = new Gtk2::RadioButton( undef, $str_insertdelete_insert );
$insert_radio->signal_connect( 'clicked', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $insert_entry = Gtk2::Entry->new_with_max_length(254);
$insert_entry->signal_connect_after( 'insert-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $insert_entry->get_position+1;} );
$insert_entry->signal_connect_after( 'delete-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $insert_entry->get_position+1;} );
my $insert_at_label = new Gtk2::Label( $str_insertdelete_at );
my $insert_adj = new Gtk2::Adjustment( 0, -256.0, 256.0, 1.0, 5.0, 0.0 );
$insert_adj->signal_connect( 'value_changed', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $insert_spin = new Gtk2::SpinButton( $insert_adj, 0, 0 );
$insert_spin->set_numeric(TRUE);

# in hbox2
my $delete_radio     = new Gtk2::RadioButton( $insert_radio, $str_insertdelete_delete );
$delete_radio->signal_connect( 'clicked', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $delete_btw1_adj  = new Gtk2::Adjustment( 0, -256.0, 256.0, 1.0, 5.0, 0.0 );
$delete_btw1_adj->signal_connect( 'value_changed', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $delete_btw1_spin = new Gtk2::SpinButton( $delete_btw1_adj, 0, 0 );
$delete_btw1_spin->set_numeric(TRUE);
my $delete_btw2_label = new Gtk2::Label( $str_insertdelete_between );
my $delete_btw2_adj   = new Gtk2::Adjustment( 1, -256.0, 256.0, 1.0, 5.0, 0.0 );
$delete_btw2_adj->signal_connect( 'value_changed', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $delete_btw2_spin  = new Gtk2::SpinButton( $delete_btw2_adj, 0, 0 );
$delete_btw2_spin->set_numeric(TRUE);

$insdel_vbox->pack_start( $insdel_hbox1,       FALSE, FALSE, 4 );
$insdel_vbox->pack_start( $insdel_hbox2,       FALSE, FALSE, 4 );
$insdel_hbox1->pack_start( $insert_radio,      FALSE, FALSE, 4 );
$insdel_hbox1->pack_start( $insert_entry,      FALSE, FALSE, 4 );
$insdel_hbox1->pack_start( $insert_at_label,   FALSE, FALSE, 4 );
$insdel_hbox1->pack_start( $insert_spin,       FALSE, FALSE, 4 );
$insdel_hbox2->pack_start( $delete_radio,      FALSE, FALSE, 4 );
$insdel_hbox2->pack_start( $delete_btw1_spin,  FALSE, FALSE, 4 );
$insdel_hbox2->pack_start( $delete_btw2_label, FALSE, FALSE, 4 );
$insdel_hbox2->pack_start( $delete_btw2_spin,  FALSE, FALSE, 4 );


######################################################################
# Replace / Remove                                                   #
######################################################################

my $reprem_vbox  = new Gtk2::VBox( FALSE, 4 );
my $reprem_hbox  = new Gtk2::HBox( FALSE, 4 );
my $reprem_label = new Gtk2::Label( $str_notebook_replaceremove );
$notebook->append_page( $reprem_vbox, $reprem_label );

my $replace_this_label = new Gtk2::Label( $str_replace );
my $replace_this_entry = Gtk2::Entry->new_with_max_length(99);
$replace_this_entry->signal_connect_after( 'insert-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $replace_this_entry->get_position+1;} );
$replace_this_entry->signal_connect_after( 'delete-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $replace_this_entry->get_position+1;} );
my $replace_with_label = new Gtk2::Label( $str_replace_with);
my $replace_with_entry = Gtk2::Entry->new_with_max_length(99);
$replace_with_entry->signal_connect_after( 'insert-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $replace_with_entry->get_position+1;} );
$replace_with_entry->signal_connect_after( 'delete-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $replace_with_entry->get_position+1;} );
my $replace_opt_case  = new Gtk2::CheckButton( $str_replace_case );
$replace_opt_case->signal_connect( 'clicked', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $replace_opt_regex = new Gtk2::CheckButton( $str_replace_regex );
$replace_opt_regex->signal_connect( 'clicked', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );

$reprem_vbox->pack_start( $reprem_hbox,        FALSE, FALSE, 4 );
$reprem_hbox->pack_start( $replace_this_label, FALSE, FALSE, 4 );
$reprem_hbox->pack_start( $replace_this_entry, FALSE, FALSE, 4 );
$reprem_hbox->pack_start( $replace_with_label, FALSE, FALSE, 4 );
$reprem_hbox->pack_start( $replace_with_entry, FALSE, FALSE, 4 );
$reprem_vbox->pack_start( $replace_opt_case,   FALSE, FALSE, 4 );
$reprem_vbox->pack_start( $replace_opt_regex,  FALSE, FALSE, 4 );


######################################################################
# Numerical                                                          #
######################################################################

my $numeric_vbox  = new Gtk2::VBox( FALSE, 4 );
my $numeric_hbox1 = new Gtk2::HBox( FALSE, 4 );
my $numeric_hbox2 = new Gtk2::HBox( FALSE, 4 );
my $numeric_hbox3 = new Gtk2::HBox( FALSE, 4 );
my $numeric_label = new Gtk2::Label( $str_notebook_numerical );
$notebook->append_page( $numeric_vbox, $numeric_label );

# in hbox1
my $numeric_start_label = new Gtk2::Label( $str_numeric_start );
my $numeric_start_adj   = new Gtk2::Adjustment( 1, 0.0, 9000.0, 1.0, 5.0, 0.0 );
$numeric_start_adj->signal_connect( 'value_changed', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $numeric_start_spin  = new Gtk2::SpinButton( $numeric_start_adj, 0, 0 );
$numeric_start_spin->set_numeric(TRUE);
my $numeric_increment_label = new Gtk2::Label( $str_numeric_increment );
my $numeric_increment_adj   = new Gtk2::Adjustment( 1, 1.0, 1000.0, 1.0, 1.0, 0.0 );
$numeric_increment_adj->signal_connect( 'value_changed', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );
my $numeric_increment_spin = new Gtk2::SpinButton( $numeric_increment_adj, 0, 0 );
$numeric_increment_spin->set_numeric(TRUE);

# in hbox2 
my $numeric_prefix_label = new Gtk2::Label( $str_numeric_prefix );
my $numeric_prefix_entry = Gtk2::Entry->new_with_max_length(254);
$numeric_prefix_entry->signal_connect_after( 'insert-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $numeric_prefix_entry->get_position+1;} );
$numeric_prefix_entry->signal_connect_after( 'delete-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $numeric_prefix_entry->get_position+1;} );
$numeric_prefix_entry->set_width_chars (9);
my $numeric_suffix_label = new Gtk2::Label( $str_numeric_suffix );
my $numeric_suffix_entry = Gtk2::Entry->new_with_max_length(254);
$numeric_suffix_entry->signal_connect_after( 'insert-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $numeric_suffix_entry->get_position+1;} );
$numeric_suffix_entry->signal_connect_after( 'delete-text', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } return '', $numeric_suffix_entry->get_position+1;} );
$numeric_suffix_entry->set_width_chars (9);

# in hbox3
my $numeric_keep_filenames = Gtk2::ComboBox->new_text;
$numeric_keep_filenames->append_text($str_numeric_keep_filenames3);
$numeric_keep_filenames->append_text($str_numeric_keep_filenames2);
$numeric_keep_filenames->append_text($str_numeric_keep_filenames1);
$numeric_keep_filenames->set_active(2);
$numeric_keep_filenames->signal_connect( 'changed', sub { if ( $settings{'auto_preview'} == TRUE ) { &preview; } } );

$numeric_vbox->pack_start(  $numeric_hbox1,           FALSE, FALSE, 4 );
$numeric_vbox->pack_start(  $numeric_hbox2,           FALSE, FALSE, 4 );
$numeric_vbox->pack_start(  $numeric_hbox3,           FALSE, FALSE, 4 );
$numeric_hbox1->pack_start( $numeric_start_label,     FALSE, FALSE, 4 );
$numeric_hbox1->pack_start( $numeric_start_spin,      FALSE, FALSE, 4 );
$numeric_hbox1->pack_start( $numeric_increment_label, FALSE, FALSE, 4 );
$numeric_hbox1->pack_start( $numeric_increment_spin,  FALSE, FALSE, 4 );
$numeric_hbox2->pack_start( $numeric_prefix_label,    FALSE, FALSE, 4 );
$numeric_hbox2->pack_start( $numeric_prefix_entry,    FALSE, FALSE, 4 );
$numeric_hbox2->pack_start( $numeric_suffix_label,    FALSE, FALSE, 4 );
$numeric_hbox2->pack_start( $numeric_suffix_entry,    FALSE, FALSE, 4 );
$numeric_hbox3->pack_start( $numeric_keep_filenames,  TRUE, TRUE, 4 );


######################################################################
# Buttons                                                            #
######################################################################

my $button_box = new Gtk2::VButtonBox();
$button_box->set_spacing_default(15);
$button_box->set_layout_default('spread');
$table->attach_defaults( $button_box, 4, 5, 0, 1 );

my $preview_button = new Gtk2::Button( $str_button_preview );
$button_box->add($preview_button);
$preview_button->set_sensitive( TRUE );
if ( $settings{'auto_preview'} == TRUE ) { $preview_button->set_sensitive( FALSE ); }
$preview_button->signal_connect( 'clicked', \&preview );

my $apply_button = new Gtk2::Button( $str_button_apply );
$button_box->add($apply_button);
$apply_button->set_sensitive( TRUE );
if ( $settings{'auto_preview'} == FALSE ) { $apply_button->set_sensitive( FALSE ); }
$apply_button->signal_connect( 'clicked', \&apply );

my $undo_button = new Gtk2::Button( $str_button_undo );
$button_box->add($undo_button);
$undo_button->set_sensitive( FALSE );
$undo_button->signal_connect( 'clicked', \&undo );

$window->show_all();
$lang_rmi_start->set_active(TRUE);
Gtk2->main;
exit(0);


######################################################################
# Functions                                                          #
######################################################################

#This function has got to be before the sub "populate_node" and "list_files_dirs"
#else it won't work.  Seems like this function have to be defined before we
#call them in the two other functions.
sub process_dir_at_iter(&;@) {
   my ( $process, $tree_model, $iter ) = @_;
   my $pathname = pathname_from_iter( $tree_model, $iter );
   if( opendir my $dh, $pathname ) {
      my @content;
      Glib::Idle->add( sub {
         my $fname = readdir $dh;
         if( defined $fname ) {
            #Next two lines are for handling accents, see http://perlmonks.org/?node_id=537023
            @content = readdir( $dh );
            $_ = decode( 'utf8', $_ ) for ( @content );
            push @content, $fname;
         }
         else {
            closedir $dh;
            $process->( $pathname, sort +( no_upwards @content ) );
            if ( $settings{'auto_preview'} == TRUE ) { &preview; }
         }
         return defined $fname;
      } );
   }
}

sub pathname_from_iter {
   my ( $tree_model, $iter ) = @_;
   my @path;
   while( $iter ) {
      unshift @path, $tree_model->get( $iter, 0 );
      $iter = $tree_model->iter_parent( $iter );
   }
   return catfile '', @path;
}

sub populate_node {
   my ( $tree_store, $node ) = @_;
   my $child;
   # remove any previously added children
   $tree_store->remove( $child ) while $child = $tree_store->iter_children( $node );
   # then put the newly read ones back in
   process_dir_at_iter {
      my $path = shift;
      my @subdir = grep { -d catfile( $path, $_ ) and -r _ } @_;
      $tree_store->set( $tree_store->append( $node ), 0 => $_ ) for @subdir;
   } $tree_store, $node;
}

sub list_files_dirs {
   my ( $tree_view ) = @_;
   @{ $filelist->{data} } = "";
   @{ $dirlist->{data}  } = "";
   process_dir_at_iter {
   my $path = shift;
   @{ $filelist->{data} } = grep ! -d catfile( $path, $_ ), @_;
   @{ $dirlist->{data}  } = grep   -d catfile( $path, $_ ), @_;
   } $tree_view->get_selection->get_selected;
   $filelist->columns_autosize;
   $dirlist->columns_autosize;
}

sub list_dir_in_tree {
   my ( $tree_view, $iter, $tree_path ) = @_;
   my $tree_model = $tree_view->get_model();
   my $child = $tree_model->iter_children( $iter );
   while ( $child ) {
      populate_node( $tree_model, $child );
      $child = $tree_model->iter_next( $child );
   }
}

sub preview {
   my $page_file_or_dir = $file_dir_notebook->get_current_page();
   my $page = $notebook->get_current_page();

   if ( defined $apply_button ) { $apply_button->set_sensitive( TRUE ); }

   if    ( $page_file_or_dir == 0 ) {
      @selected_items = $filelist->get_selected_indices;
      if    ( $page == 0 ) { &case    ( $filelist->{data} ); }
      elsif ( $page == 1 ) { &insdel  ( $filelist->{data} ); }
      elsif ( $page == 2 ) { &replace ( $filelist->{data} ); }
      elsif ( $page == 3 ) { &numeric ( $filelist->{data} ); }
   }
   elsif ( $page_file_or_dir == 1 ) {
      @selected_items = $dirlist->get_selected_indices;
      if    ( $page == 0 ) { &case    ( $dirlist->{data} ); }
      elsif ( $page == 1 ) { &insdel  ( $dirlist->{data} ); }
      elsif ( $page == 2 ) { &replace ( $dirlist->{data} ); }
      elsif ( $page == 3 ) { &numeric ( $dirlist->{data} ); }
   }
}

sub apply {
   #To know if we rename files or folders (0=files and 1=folders)
   my $page_file_or_dir = $file_dir_notebook->get_current_page();

   #This is to rename files
   if ( $page_file_or_dir == 0 ) { &rename_now( $filelist->{data} ); }

   #This is to rename folders
   if ( $page_file_or_dir == 1 ) { &rename_now( $dirlist ->{data} ); }

   $filelist->columns_autosize;
   $dirlist->columns_autosize;

   &check_log_size;
}

sub rename_now {
   my @file_or_dir_list = @_;
   my $name_old;
   my $name_new;
   my $name_tmp;
   my $name_tmp2;
   my $i=0;
   my $j=0;
   my $i_undo=0;
   my $rename=0;
   $exist=0;
   $max_length=0;
   $max_length_undo=0;
   @undo_oldname=();
   @undo_newname=();
   @selection=();
   my $page_file_or_dir = $file_dir_notebook->get_current_page();
   if ( $page_file_or_dir == 0 ) { @selection = $filelist->get_selected_indices; }
   if ( $page_file_or_dir == 1 ) { @selection = $dirlist ->get_selected_indices; }

   #To know the path we are in right now (comes without a trailing slash)
   my $path = pathname_from_iter( $dirtree->get_selection->get_selected );
   $path =~ s/\/\//\//g; #replace // to / in the path
   
   for ( 0..$#{$file_or_dir_list[0]} ) {
      if ( defined $file_or_dir_list[0][$i][0] and defined $file_or_dir_list[0][$i][1] and $file_or_dir_list[0][$i][0] ne $file_or_dir_list[0][$i][1] and $file_or_dir_list[0][$i][1] ne "" ) {
         $name_tmp2 = $file_or_dir_list[0][$i][1];

         #Check if a file or a folder already exist with the name we need
         if ( $security == 1 and $exist == 0 and -e "$path/$file_or_dir_list[0][$i][1]" ) {
            my $dialog = Gtk2::MessageDialog->new ($window, 'modal', 'info', 'ok', decode('utf8', "$str_exists1\n\n$str_list_filename : $path/$file_or_dir_list[0][$i][0]\n$str_list_newname : $path/$file_or_dir_list[0][$i][1]\n"));
            $dialog->run;
            $dialog->destroy;
            $exist=1;
         }

         if ( $exist == 0 ) {
            # if the setting Trim Spaces is checked, then trim it
            if ( $settings{'trim_spaces'} == TRUE ) { 
               $name_tmp2 =~ s/^\s+//;     #trim space at start
               $name_tmp2 =~ s/\s+$//;     #trim space at end
               $name_tmp2 =~ s/\s{2,}/ /g; #replace more than 2 spaces to only 1
            }

            #We have to rename the files/dirs 2 times with "_tmp_gprename" because you can't
            #rename the file "a" to "A" in one shot on a Windows filesystem
            $name_old = "$path/$file_or_dir_list[0][$i][0]";
            $name_tmp = "$path/$name_tmp2._tmp_gprename";
            $name_new = "$path/$name_tmp2";
            rename( $name_old, $name_tmp );
            rename( $name_tmp, $name_new );

            $undo_oldname[$i_undo] = $name_old;
            $undo_newname[$i_undo] = $name_new;
            $i_undo++;

            $rename=1;

            if ( $max_length      < length($file_or_dir_list[0][$i][0]) ) { $max_length      = length($file_or_dir_list[0][$i][0]); }
            if ( $max_length_undo < length($file_or_dir_list[0][$i][1]) ) { $max_length_undo = length($file_or_dir_list[0][$i][1]); }

         }# end if exist == 0
      }
   $i++;
   }# end for

   # if there was a rename with no error then print to the log file
   if ( $rename == 1 and $exist == 0 ) {
      open( OUTPUT, '>>', $log_file ) or die "Couldn't open $log_file for writing: $!\n";
      flock( OUTPUT, LOCK_EX );
      print OUTPUT localtime() . " - $path\n";

      $i=0;
      while ( $i < @undo_newname ) {
         print OUTPUT substr($undo_oldname[$i], rindex($undo_oldname[$i],'/')+1);
         $j=length(decode('utf8',substr($undo_oldname[$i], rindex($undo_oldname[$i],'/')+1)));
         while ( $j < $max_length ) { print OUTPUT " "; $j++; }
         print OUTPUT " => " . substr($undo_newname[$i], rindex($undo_newname[$i],'/')+1) . "\n";
         $i++;
      }

      print OUTPUT "\n";
      flock( OUTPUT, LOCK_UN );
      close( OUTPUT );
      $undo_button->set_sensitive( TRUE );
   }
   # if there was a renaming and there was an error then call undo
   elsif ( $rename == 1 and $exist == 1 ) {
      &undo; 
   }

   # clear the New name column
   $i=0;
   if ( $settings{'auto_preview'} == FALSE ) { for ( 0..$#{$file_or_dir_list[0]} ) { $file_or_dir_list[0][$i][1] = ""; $i++; } }

   # if there was an error, keep the selection of the user
   if   ( $exist == 1 ) { $filelist->select(@selection); }
   else                 { &list_files_dirs($dirtree);    }

   if ( $settings{'auto_preview'} == FALSE ) { $apply_button->set_sensitive( FALSE ); }
}

sub case {
   my @file_or_dir_list = @_;
   my $case_first_after = $case_first_after_entry->get_chars( 0, -1 );
   my @case_first_after = split( /\s{0}/, $case_first_after );
   my $frag;
   my $name_tmp;
   my $i=0;
   my $j;

   for ( 0..$#{$file_or_dir_list[0]} ) {
      if    ( $case_all_up_radio->get_active ) { $name_tmp = "\U$file_or_dir_list[0][$i][0]"; }
      elsif ( $case_all_lo_radio->get_active ) { $name_tmp = "\L$file_or_dir_list[0][$i][0]"; }
      else {
         my $temp = "\L\u$file_or_dir_list[0][$i][0]";
         foreach (@case_first_after) {
            my @temp = split( /\Q$_/, $temp );
            foreach $frag (@temp) { $frag = "\u$frag"; }
            $temp = join( "$_", @temp );
         }
         $name_tmp = $temp;
      }

      # clear the new name column and add the new name if it is selected or not
      $file_or_dir_list[0][$i][1] = '';
      $name_tmp = &options_trim_spaces_and_return($name_tmp);
      if ( @{selected_items} == FALSE ) { $file_or_dir_list[0][$i][1] = $name_tmp; }
      else { foreach $j (@selected_items) { if ( $i == $j ) { $file_or_dir_list[0][$i][1] = $name_tmp; } } }

   $i++;
   }
}

sub insdel {
   my @file_or_dir_list = @_;
   my $insert_str = '';
   my $insert_at = 0;
   $insert_str = $insert_entry->get_chars( 0, -1 );
   $insert_at  = $insert_spin->get_value_as_int();
   my $delete_from = 0;
   my $delete_to = 0;
   $delete_from = $delete_btw1_spin->get_value_as_int();
   $delete_to = $delete_btw2_spin->get_value_as_int();
   my $i=0;
   my $j=0;
   my $name_tmp='';
   my $tmp_string1='';
   my $tmp_string2='';
   my $tmp_number=0;


   for ( 0..$#{$file_or_dir_list[0]} ) {
      if ( $insert_radio->get_active ) {
         $insert_at  = $insert_spin->get_value_as_int(); # have to reset it because it will change next line
         if ( $insert_at >  length($file_or_dir_list[0][$i][0]) ) { $insert_at = length($file_or_dir_list[0][$i][0]); } # for no warning of "substr outside of string"
         $tmp_string1 = substr ( $file_or_dir_list[0][$i][0], 0, $insert_at );
         $tmp_string2 = substr ( $file_or_dir_list[0][$i][0], $insert_at, length($file_or_dir_list[0][$i][0]) );
         $name_tmp = $tmp_string1 . $insert_str . $tmp_string2;
      }
      elsif ( $delete_radio->get_active ) {
         $delete_to = $delete_btw2_spin->get_value_as_int(); # have to reset it because it will change next line
         if ( $delete_to >  length($file_or_dir_list[0][$i][0]) ) { $delete_to = length($file_or_dir_list[0][$i][0]); } # for no warning of "substr outside of string"
         $tmp_string1 = substr ( $file_or_dir_list[0][$i][0], 0, $delete_from );
         $tmp_string2 = substr ( $file_or_dir_list[0][$i][0], $delete_to, length($file_or_dir_list[0][$i][0]) );
         $name_tmp = $tmp_string1 . $tmp_string2;
      }

      # clear the new name column and add the new name if it is selected or not
      $file_or_dir_list[0][$i][1] = '';
      $name_tmp = &options_trim_spaces_and_return($name_tmp);
      if ( @{selected_items} == FALSE ) { $file_or_dir_list[0][$i][1] = $name_tmp; }
      else { foreach $j (@selected_items) { if ( $i == $j ) { $file_or_dir_list[0][$i][1] = $name_tmp; } } }
   $i++;
   }
}

sub replace {
   my @file_or_dir_list = @_;
   my $replace_this = '';
   $replace_this = $replace_this_entry->get_chars( 0, -1 );
   my $replace_with = '';
   $replace_with = $replace_with_entry->get_chars( 0, -1 );
   my $name_tmp='';
   my $i=0;
   my $j=0;

   for ( 0..$#{$file_or_dir_list[0]} ) {
      $name_tmp = $file_or_dir_list[0][$i][0];

      # Substitution with pattern metacharacters
      if ( $replace_opt_regex->get_active ) {
         if ( $replace_opt_case->get_active ) { $name_tmp =~ s/$replace_this/$replace_with/g; }
         else                                 { $name_tmp =~ s/$replace_this/$replace_with/ig; }
      }

      # Substitution without pattern metacharacters (hence the \Q)
      else {
         if ( $replace_opt_case->get_active ) { $name_tmp =~ s/\Q$replace_this/$replace_with/g;  }
         else                                 { $name_tmp =~ s/\Q$replace_this/$replace_with/ig; }
      }

      # clear the new name column and add the new name if it is selected or not
      $file_or_dir_list[0][$i][1] = '';
      $name_tmp = &options_trim_spaces_and_return($name_tmp);
      if ( @{selected_items} == FALSE ) { $file_or_dir_list[0][$i][1] = $name_tmp; }
      else { foreach $j (@selected_items) { if ( $i == $j ) { $file_or_dir_list[0][$i][1] = $name_tmp; } } }
   $i++;
   }
}

sub numeric {
   my @file_or_dir_list = @_;
   my $numeric_start     = $numeric_start_spin->get_value_as_int();
   my $numeric_increment = $numeric_increment_spin->get_value_as_int();
   my $numeric_prefix    = '';
   $numeric_prefix = $numeric_prefix_entry->get_chars( 0, -1 );
   my $numeric_suffix    = '';
   $numeric_suffix = $numeric_suffix_entry->get_chars( 0, -1 );
   my $i=0;
   my $j=0;
   my $k=0;
   my $max_no = 0;;
   my $max_chrs = 0;
   my $name_tmp='';
   my $numeric_number=0;

   # Get the number of items to rename, selected or not
   for ( 0..$#{$file_or_dir_list[0]} ) {
      if ( @{selected_items} == FALSE ) { $max_no++; }
      else { foreach $j (@selected_items) { if ( $i == $j ) { $max_no++; } } }
   $i++;
   }
   $max_no = ($max_no - 1) * $numeric_increment + $numeric_start;
   $max_chrs = length($max_no);
   $k = $numeric_start;

   $i=0;
   for ( 0..$#{$file_or_dir_list[0]} ) {
      # Calculate and print the right numbers of 0 if zero_autofill is checked
      $numeric_number = $k;
      if ( $zero_autofill_rmi->get_active ) {
         while ( length( $numeric_number ) < $max_chrs ) {
            $numeric_number = 0 . $numeric_number;
         }
      }

      # Set $name_tmp with the text positioned at the right place depending of $numeric_keep_filenames
      if (  $numeric_keep_filenames->get_active == 0  ) {
         $name_tmp = $numeric_prefix . $numeric_number . $numeric_suffix . $file_or_dir_list[0][$i][0];
      }
      elsif (  $numeric_keep_filenames->get_active == 1 ) {
         $name_tmp = $file_or_dir_list[0][$i][0] . $numeric_prefix . $numeric_number . $numeric_suffix;
      }
      else { $name_tmp = $numeric_prefix . $numeric_number . $numeric_suffix; }

      # clear the new name column and add the new name if it is selected or not
      $file_or_dir_list[0][$i][1] = '';
      $name_tmp = &options_trim_spaces_and_return($name_tmp);
      if ( @{selected_items} == FALSE ) { 
         $file_or_dir_list[0][$i][1] = $name_tmp;
         $k = $k + $numeric_increment;
      }
      else { 
         foreach $j (@selected_items) { 
            if ( $i == $j ) { 
               $file_or_dir_list[0][$i][1] = $name_tmp;
               $k = $k + $numeric_increment;
            } 
         } 
      }

   $i++;
   }
}

sub check_log_size {
   if ( -s $log_file > 100000 ) {
      my $dialog = Gtk2::MessageDialog->new ($window, 'modal', 'info', 'ok', decode('utf8',$str_log_size) );
      $dialog->run;
      $dialog->destroy;
   }
}

sub quit {
   &save_settings;
   exit(0);
}

#sub recursive {
#   if   ( $recursive_rmi->get_active ) { $recursive = TRUE;  }
#   else {                                $recursive = FALSE; }
#}

sub options_trim_spaces {
   if   ( $trim_spaces_rmi->get_active ) { $settings{'trim_spaces'} = TRUE;  }
   else {                                  $settings{'trim_spaces'} = 0; }
   &preview;
}

sub options_trim_spaces_and_return {
   my @i = @_;

   if ( $settings{'trim_spaces'} == TRUE ) { 
      $i[0] =~ s/^\s+//;
      $i[0] =~ s/\s+$//;
      $i[0] =~ s/\s{2,}/ /g;
   }

   return $i[0];
}

sub options_zero_autofill {
   if   ( $zero_autofill_rmi->get_active ) { $settings{'zero_autofill'} = TRUE; &preview; }
   else {                                    $settings{'zero_autofill'} = 0;    &preview; }
}

sub options_fullscreen {
   if   ( $fullscreen_rmi->get_active ) { $settings{'fullscreen'} = TRUE;  $window->fullscreen;   }
   else {                                 $settings{'fullscreen'} = 0;     $window->unfullscreen; }
}

sub options_auto_preview {
   if ( $auto_preview_rmi->get_active ) { 
      $settings{'auto_preview'} = TRUE;  
      $preview_button->set_sensitive(FALSE); 
      $apply_button->set_sensitive( TRUE );
      &preview;
   }
   else {                                   
      $settings{'auto_preview'} = 0; 
      $preview_button->set_sensitive(TRUE);
      $apply_button->set_sensitive( FALSE );
      @{ $filelist->{data} } = '';
      @{ $dirlist->{data}  } = '';
      &list_files_dirs($dirtree); 
   }
}

sub options_view_log {
   # Read the file .gprename_log and put everything in a label
   my $log_label = new Gtk2::Label( '' );
   $log_label->set_justify('left');
   open( INPUT, '<', $log_file ) or die "Couldn't open $log_file for reading: $!\n";
   flock( INPUT, LOCK_EX );
   while (my $line = <INPUT>) { $log_label->set_label($log_label->get_label . $line); }
   flock( INPUT, LOCK_UN );
   close( INPUT );

   my $dialog = Gtk2::Dialog->new ('', $window, [qw/modal destroy-with-parent/], $str_menu_log_clear => 'accept', 'gtk-ok' => 'reject');
   $dialog->set_has_separator (TRUE);
   my $log_sw = new Gtk2::ScrolledWindow( undef, undef );
   $log_sw->set_policy( 'automatic', 'automatic' );
   $log_sw->set_shadow_type ('in');

   # Put the label in a VBox and the VBox in a HBox and the HBox in a Scrolled Window
   # so that the label can be positioned at the top, left.
   my $vbox = Gtk2::VBox->new (0, 0);
   my $hbox = Gtk2::HBox->new (0, 0);
   $vbox->pack_start($log_label, FALSE, FALSE, FALSE);
   $hbox->pack_start($vbox, FALSE, FALSE, FALSE);
   $log_sw->add_with_viewport( $hbox );
   $dialog->vbox->pack_start($log_sw, TRUE, TRUE, TRUE);
   $dialog->set_default_size ($settings{'width'}, $settings{'height'});

   $vbox->show();
   $hbox->show();
   $log_sw->show();
   $log_label->show();

   $dialog->signal_connect(response => sub {
      if ( $_[1] =~ m/accept/ ) {
         open( INPUT, '>', $log_file ) or die "Couldn't open $log_file for reading: $!\n";
         flock( INPUT, LOCK_EX );
         print INPUT '';
         flock( INPUT, LOCK_UN );
         close( INPUT );
         $log_label->set_label("");
      }
      elsif ( $_[1] =~ m/reject/ ) { $dialog->destroy; }
   });

   $dialog->run;
}

sub options_security {
   if   ( $security_rmi->get_active ) { $settings{'security'} = 0;    $security=0; }
   else {                               $settings{'security'} = TRUE; $security=1; }
}

sub undo {
   my $i = @undo_newname; #get the number of items
   $i--;
   my $j;
   my $path = $undo_newname[0];
   my $max = $max_length;
   if ( $max < $max_length_undo ) { $max = $max_length_undo; }
   $path = substr($path, 0, rindex($path, '/') ); #extract only the path

   open( OUTPUT, '>>', $log_file ) or die "Couldn't open $log_file for writing: $!\n";
   flock( OUTPUT, LOCK_EX );
   print OUTPUT localtime() . " - $path\n";

   # We rename in reverse order to make sure we don't overwrite files/dirs
   while ( $i >= 0 ) {
      rename( $undo_newname[$i], $undo_newname[$i] . '_tmp_gprename' );
      rename( $undo_newname[$i] . '_tmp_gprename', $undo_oldname[$i] );
      $i--;
   }

   # Then we print to the log file in sorted order
   $i=0;
   while ( $i < @undo_newname ) {
      print OUTPUT substr($undo_newname[$i], rindex($undo_newname[$i],'/')+1);
      $j=length(decode('utf8',substr($undo_newname[$i], rindex($undo_newname[$i],'/')+1)));
      while ( $j < $max ) { print OUTPUT " "; $j++; }
      print OUTPUT " => " . substr($undo_oldname[$i], rindex($undo_oldname[$i],'/')+1) . "\n";
      $i++;
   }

   $undo_button->set_sensitive(FALSE);
   if ( $exist == 0 ) { &list_files_dirs($dirtree); }
   print OUTPUT "\n";
   flock( OUTPUT, LOCK_UN );
   close( OUTPUT );
}

sub read_settings {
   if ( !-e $setting_file ) { return; }
   open( INPUT, '<', $setting_file ) or die "Couldn't open $setting_file for reading: $!\n";
   flock( INPUT, LOCK_EX );
   while (my $line = <INPUT>) {
      chomp($line);
      (my $key, my $value) = split(/=/, $line);
      $settings{$key} = $value;
   }
   flock( INPUT, LOCK_UN );
   close( INPUT );
}

sub save_settings {
   my $width_tmp;
   my $height_tmp;
   ($width_tmp, $height_tmp) = $window->get_size;
   $settings{'width'} = $width_tmp;
   $settings{'height'} = $height_tmp;

   my $key;
   open( OUTPUT, '>', $setting_file ) or die "Couldn't open $setting_file for writing: $!\n";
   flock( OUTPUT, LOCK_EX );
   foreach $key ( sort keys %settings ) { print OUTPUT $key, '=', $settings{$key}, "\n"; }
   flock( OUTPUT, LOCK_UN );
   close( OUTPUT );
}

sub language_selected {
   my ( $selected ) = @_;
   $settings{'lang'} = $selected->child->get_label;
   &save_settings;
   do "$install_dir/languages/$langs{$settings{'lang'}}" ;

   $apply_button     ->child->set_label( decode('utf8', $str_button_apply           ));
   $preview_button   ->child->set_label( decode('utf8', $str_button_preview         ));
   $undo_button      ->child->set_label( decode('utf8', $str_button_undo            ));
   $case_all_lo_radio->child->set_label( decode('utf8', $str_case_all_lo            ));
   $case_all_up_radio->child->set_label( decode('utf8', $str_case_all_up            ));
   $case_first_radio ->child->set_label( decode('utf8', $str_case_first             ));
   $insert_at_label         ->set_label( decode('utf8', $str_insertdelete_at        ));
   $delete_btw2_label       ->set_label( decode('utf8', $str_insertdelete_between   ));
   $delete_radio     ->child->set_label( decode('utf8', $str_insertdelete_delete    ));
   $insert_radio     ->child->set_label( decode('utf8', $str_insertdelete_insert    ));
   $about_mi         ->child->set_label( decode('utf8', $str_menu_about             ));
   $file_dir_notebook->set_tab_label_text ($file_dir_notebook->get_nth_page(1), decode('utf8',$str_menu_directory));
   $file_mm          ->child->set_label( decode('utf8', $str_menu_file              ));
   $str_menu_file =~ s/_//;
   $file_dir_notebook->set_tab_label_text ($file_dir_notebook->get_nth_page(0), decode('utf8',$str_menu_file));
   $help_mm          ->child->set_label( decode('utf8', $str_menu_help              ));
   $lang_mm          ->child->set_label( decode('utf8', $str_menu_lang              ));
   $options_mi       ->child->set_label( decode('utf8', $str_menu_options           ));
   $quit_mi          ->child->set_label( decode('utf8', $str_menu_quit              ));
   $tips_mi          ->child->set_label( decode('utf8', $str_menu_tips              ));
   $view_log_mi      ->child->set_label( decode('utf8', $str_menu_log               ));
   $case_label              ->set_label( decode('utf8', $str_notebook_casechange    ));
   $insdel_label            ->set_label( decode('utf8', $str_notebook_insertdelete  ));
   $numeric_label           ->set_label( decode('utf8', $str_notebook_numerical     ));
   $reprem_label            ->set_label( decode('utf8', $str_notebook_replaceremove ));
   $numeric_increment_label ->set_label( decode('utf8', $str_numeric_increment      ));
   $numeric_prefix_label    ->set_label( decode('utf8', $str_numeric_prefix         ));
   $numeric_start_label     ->set_label( decode('utf8', $str_numeric_start          ));
   $numeric_suffix_label    ->set_label( decode('utf8', $str_numeric_suffix         ));
   $auto_preview_rmi ->child->set_label( decode('utf8', $str_options_auto_preview   ));
   $fullscreen_rmi   ->child->set_label( decode('utf8', $str_options_fullscreen     ));
#   $recursive_rmi    ->child->set_label( decode('utf8', $str_options_recursively    ));
   $security_rmi     ->child->set_label( decode('utf8', $str_options_security       ));
   $trim_spaces_rmi  ->child->set_label( decode('utf8', $str_options_trim_spaces    ));
   $zero_autofill_rmi->child->set_label( decode('utf8', $str_options_zero_autofill  ));
   $replace_opt_case ->child->set_label( decode('utf8', $str_replace_case           ));
   $replace_opt_regex->child->set_label( decode('utf8', $str_replace_regex          ));
   $replace_this_label      ->set_label( decode('utf8', $str_replace                ));
   $replace_with_label      ->set_label( decode('utf8', $str_replace_with           ));
   $numeric_keep_filenames->remove_text(2);
   $numeric_keep_filenames->remove_text(1);
   $numeric_keep_filenames->remove_text(0);
   $numeric_keep_filenames->append_text(decode('utf8', $str_numeric_keep_filenames3));
   $numeric_keep_filenames->append_text(decode('utf8', $str_numeric_keep_filenames2));
   $numeric_keep_filenames->append_text(decode('utf8', $str_numeric_keep_filenames1));
   $numeric_keep_filenames->set_active(2);
}

sub help_tips {
   my $label = new Gtk2::Label( decode('utf8', $str_tips) );
   my $dialog = Gtk2::Dialog->new ($str_menu_tips, $window, [qw/modal destroy-with-parent/], 'gtk-ok' => 'accept');
   $dialog->set_has_separator (TRUE);
   my $about_sw = new Gtk2::ScrolledWindow( undef, undef );
   $about_sw->set_policy( 'automatic', 'automatic' );
   $about_sw->set_shadow_type ('in');

   # Put the label in a VBox and the VBox in a HBox and the HBox in a Scrolled Window
   # so that the label can be positioned at the top, left.
   my $vbox = Gtk2::VBox->new (0, 0);
   my $hbox = Gtk2::HBox->new (0, 0);
   $vbox->pack_start($label, FALSE, FALSE, FALSE);
   $hbox->pack_start($vbox, FALSE, FALSE, FALSE);
   $about_sw->add_with_viewport( $hbox );
   $dialog->vbox->pack_start($about_sw, TRUE, TRUE, TRUE);
   $dialog->set_default_size (550, 400);

   $vbox->show();
   $hbox->show();
   $about_sw->show();
   $label->show();

   $dialog->run;
   $dialog->destroy;
}

sub help_about {
   # Read the file COPYING and put it in a label
   my $label = new Gtk2::Label( '' );
   $label->set_label("$str_gprename_version\n\n");

   open( INPUT, '<', "$install_dir/COPYING" ) or die "Couldn't open $install_dir/COPYING for reading: $!\n";
   flock( INPUT, LOCK_EX );
   while (my $line = <INPUT>) { $label->set_label($label->get_label . decode('utf8',$line)); }
   flock( INPUT, LOCK_UN );
   close( INPUT );

   my $dialog = Gtk2::Dialog->new ("", $window, [qw/modal destroy-with-parent/], 'gtk-ok' => 'accept');
   $dialog->set_has_separator (TRUE);
   my $about_sw = new Gtk2::ScrolledWindow( undef, undef );
   $about_sw->set_policy( 'automatic', 'automatic' );
   $about_sw->set_shadow_type ('in');

   # Put the label in a VBox and the VBox in a HBox and the HBox in a Scrolled Window
   # so that the label can be positioned at the top, left.
   my $vbox = Gtk2::VBox->new (0, 0);
   my $hbox = Gtk2::HBox->new (0, 0);
   $vbox->pack_start($label, FALSE, FALSE, FALSE);
   $hbox->pack_start($vbox, FALSE, FALSE, FALSE);
   $about_sw->add_with_viewport( $hbox );
   $dialog->vbox->pack_start($about_sw, TRUE, TRUE, TRUE);
   $dialog->set_default_size (640, 420);

   $vbox->show();
   $hbox->show();
   $about_sw->show();
   $label->show();

   $dialog->run;
   $dialog->destroy;
}
